@@ -20,6 +20,7 @@ from random import random
 
 import swift.common.db
 from swift.account.backend import AccountBroker, DATADIR
+from swift.common.exceptions import InvalidAccountInfo
 from swift.common.utils import get_logger, audit_location_generator, \
     config_true_value, dump_recon_cache, ratelimit_sleep
 from swift.common.daemon import Daemon
@@ -30,12 +31,13 @@ from eventlet import Timeout
 class AccountAuditor(Daemon):
     """Audit accounts."""
 
-    def __init__(self, conf):
+    def __init__(self, conf, logger=None):
         self.conf = conf
-        self.logger = get_logger(conf, log_route='account-auditor')
+        self.logger = logger or get_logger(conf, log_route='account-auditor')
         self.devices = conf.get('devices', '/srv/node')
         self.mount_check = config_true_value(conf.get('mount_check', 'true'))
         self.interval = int(conf.get('interval', 1800))
+        self.logging_interval = 3600  # once an hour
         self.account_passes = 0
         self.account_failures = 0
         self.accounts_running_time = 0
@@ -53,7 +55,7 @@ class AccountAuditor(Daemon):
                                             logger=self.logger)
         for path, device, partition in all_locs:
             self.account_audit(path)
-            if time.time() - reported >= 3600:  # once an hour
+            if time.time() - reported >= self.logging_interval:
                 self.logger.info(_('Since %(time)s: Account audits: '
                                    '%(passed)s passed audit,'
                                    '%(failed)s failed audit'),
@@ -103,6 +105,28 @@ class AccountAuditor(Daemon):
         dump_recon_cache({'account_auditor_pass_completed': elapsed},
                          self.rcache, self.logger)
 
+    def validate_per_policy_counts(self, broker):
+        info = broker.get_info()
+        policy_stats = broker.get_policy_stats(do_migrations=True)
+        policy_totals = {
+            'container_count': 0,
+            'object_count': 0,
+            'bytes_used': 0,
+        }
+        for policy_stat in policy_stats.values():
+            for key in policy_totals:
+                policy_totals[key] += policy_stat[key]
+
+        for key in policy_totals:
+            if policy_totals[key] == info[key]:
+                continue
+            raise InvalidAccountInfo(_(
+                'The total %(key)s for the container (%(total)s) does not '
+                'match the sum of %(key)s across policies (%(sum)s)')
+                % {'key': key,
+                   'total': info[key],
+                   'sum': policy_totals[key]})
+
     def account_audit(self, path):
         """
         Audits the given account path
@@ -113,10 +137,15 @@ class AccountAuditor(Daemon):
         try:
             broker = AccountBroker(path)
             if not broker.is_deleted():
-                broker.get_info()
+                self.validate_per_policy_counts(broker)
                 self.logger.increment('passes')
                 self.account_passes += 1
-                self.logger.debug(_('Audit passed for %s') % broker)
+                self.logger.debug('Audit passed for %s' % broker)
+        except InvalidAccountInfo as e:
+            self.logger.increment('failures')
+            self.account_failures += 1
+            self.logger.error(
+                _('Audit Failed for %s: %s'), path, str(e))
         except (Exception, Timeout):
             self.logger.increment('failures')
             self.account_failures += 1
