@@ -191,6 +191,7 @@ class AccountController(BaseStorageServer):
             return HTTPPreconditionFailed(body='Bad delimiter')
         limit = constraints.ACCOUNT_LISTING_LIMIT
         given_limit = get_param(req, 'limit')
+        reverse = config_true_value(get_param(req, 'reverse'))
         if given_limit and given_limit.isdigit():
             limit = int(given_limit)
             if limit > constraints.ACCOUNT_LISTING_LIMIT:
@@ -211,7 +212,7 @@ class AccountController(BaseStorageServer):
             return self._deleted_response(broker, req, HTTPNotFound)
         return account_listing_response(account, req, out_content_type, broker,
                                         limit, marker, end_marker, prefix,
-                                        delimiter)
+                                        delimiter, reverse)
 
     @public
     @replication
