@@ -40,9 +40,10 @@ The filter returns HTTPBadRequest if path is invalid.
 @author: eamonn-otoole
 '''
 
+from six.moves.urllib.parse import unquote
+
 import re
 from swift.common.utils import get_logger
-from urllib2 import unquote
 
 from swift.common.swob import Request, HTTPBadRequest
 
@@ -59,7 +60,7 @@ class NameCheckMiddleware(object):
         self.conf = conf
         self.forbidden_chars = self.conf.get('forbidden_chars',
                                              FORBIDDEN_CHARS)
-        self.maximum_length = self.conf.get('maximum_length', MAX_LENGTH)
+        self.maximum_length = int(self.conf.get('maximum_length', MAX_LENGTH))
         self.forbidden_regexp = self.conf.get('forbidden_regexp',
                                               FORBIDDEN_REGEXP)
         if self.forbidden_regexp:
@@ -78,12 +79,7 @@ class NameCheckMiddleware(object):
         self.logger.debug("name_check: self.forbidden_chars %s" %
                           self.forbidden_chars)
 
-        for c in unquote(req.path):
-            if c in self.forbidden_chars:
-                return True
-            else:
-                pass
-        return False
+        return any((c in unquote(req.path)) for c in self.forbidden_chars)
 
     def check_length(self, req):
         '''
@@ -92,10 +88,7 @@ class NameCheckMiddleware(object):
         Returns False if the length is <= the maximum
         '''
         length = len(unquote(req.path))
-        if length > self.maximum_length:
-            return True
-        else:
-            return False
+        return length > self.maximum_length
 
     def check_regexp(self, req):
         '''
@@ -120,18 +113,20 @@ class NameCheckMiddleware(object):
         if self.check_character(req):
             return HTTPBadRequest(
                 request=req,
-                body=("Object/Container name contains forbidden chars from %s"
+                body=("Object/Container/Account name contains forbidden "
+                      "chars from %s"
                       % self.forbidden_chars))(env, start_response)
         elif self.check_length(req):
             return HTTPBadRequest(
                 request=req,
-                body=("Object/Container name longer than the allowed maximum "
+                body=("Object/Container/Account name longer than the "
+                      "allowed maximum "
                       "%s" % self.maximum_length))(env, start_response)
         elif self.check_regexp(req):
             return HTTPBadRequest(
                 request=req,
-                body=("Object/Container name contains a forbidden substring "
-                      "from regular expression %s"
+                body=("Object/Container/Account name contains a forbidden "
+                      "substring from regular expression %s"
                       % self.forbidden_regexp))(env, start_response)
         else:
             # Pass on to downstream WSGI component
