@@ -57,12 +57,11 @@ The format of the list will be:
       "range": "1048576-2097151"}, ...]
 
 The number of object segments is limited to a configurable amount, default
-1000. Each segment, except for the final one, must be at least 1 megabyte
-(configurable). On upload, the middleware will head every segment passed in to
-verify:
+1000. Each segment must be at least 1 byte. On upload, the middleware will
+head every segment passed in to verify:
 
  1. the segment exists (i.e. the HEAD was successful);
- 2. the segment meets minimum size requirements (if not the last segment);
+ 2. the segment meets minimum size requirements;
  3. if the user provided a non-null etag, the etag matches;
  4. if the user provided a non-null size_bytes, the size_bytes matches; and
  5. if the user provided a range, it is a singular, syntactically correct range
@@ -121,8 +120,9 @@ finally bytes 2095104 through 2097152 (i.e., the last 2048 bytes) of
 
   .. note::
 
-     The minimum sized range is min_segment_size, which by
-     default is 1048576 (1MB).
+
+     The minimum sized range is 1 byte. This is the same as the minimum
+     segment size.
 
 
 -------------------------
@@ -149,9 +149,15 @@ A GET request with the query parameter::
 
     ?multipart-manifest=get
 
-Will return the actual manifest file itself. This is generated json and does
-not match the data sent from the original multipart-manifest=put. This call's
-main purpose is for debugging.
+will return a transformed version of the original manifest, containing
+additional fields and different key names.
+
+A GET request with the query parameters::
+
+    ?multipart-manifest=get&format=raw
+
+will return the contents of the original manifest as it was sent by the client.
+The main purpose for both calls is solely debugging.
 
 When the manifest object is uploaded you are more or less guaranteed that
 every segment in the manifest exists and matched the specifications.
@@ -221,7 +227,7 @@ from swift.common.middleware.bulk import get_response_body, \
     ACCEPTABLE_FORMATS, Bulk
 
 
-DEFAULT_MIN_SEGMENT_SIZE = 1024 * 1024  # 1 MiB
+DEFAULT_RATE_LIMIT_UNDER_SIZE = 1024 * 1024  # 1 MiB
 DEFAULT_MAX_MANIFEST_SEGMENTS = 1000
 DEFAULT_MAX_MANIFEST_SIZE = 1024 * 1024 * 2  # 2 MiB
 
@@ -231,7 +237,7 @@ OPTIONAL_SLO_KEYS = set(['range'])
 ALLOWED_SLO_KEYS = REQUIRED_SLO_KEYS | OPTIONAL_SLO_KEYS
 
 
-def parse_and_validate_input(req_body, req_path, min_segment_size):
+def parse_and_validate_input(req_body, req_path):
     """
     Given a request body, parses it and returns a list of dictionaries.
 
@@ -269,7 +275,6 @@ def parse_and_validate_input(req_body, req_path, min_segment_size):
     vrs, account, _junk = split_path(req_path, 3, 3, True)
 
     errors = []
-    num_segs = len(parsed_data)
     for seg_index, seg_dict in enumerate(parsed_data):
         if not isinstance(seg_dict, dict):
             errors.append("Index %d: not a JSON object" % seg_index)
@@ -315,10 +320,10 @@ def parse_and_validate_input(req_body, req_path, min_segment_size):
             except (TypeError, ValueError):
                 errors.append("Index %d: invalid size_bytes" % seg_index)
                 continue
-            if (seg_size < min_segment_size and seg_index < num_segs - 1):
-                errors.append("Index %d: too small; each segment, except "
-                              "the last, must be at least %d bytes."
-                              % (seg_index, min_segment_size))
+            if seg_size < 1:
+                errors.append("Index %d: too small; each segment must be "
+                              "at least 1 byte."
+                              % (seg_index,))
                 continue
 
         obj_path = '/'.join(['', vrs, account, seg_dict['path'].lstrip('/')])
@@ -574,6 +579,9 @@ class SloGetContext(WSGIContext):
 
         # Handle pass-through request for the manifest itself
         if req.params.get('multipart-manifest') == 'get':
+            if req.params.get('format') == 'raw':
+                resp_iter = self.convert_segment_listing(
+                    self._response_headers, resp_iter)
             new_headers = []
             for header, value in self._response_headers:
                 if header.lower() == 'content-type':
@@ -607,7 +615,40 @@ class SloGetContext(WSGIContext):
             req, resp_headers, resp_iter)
         return response(req.environ, start_response)
 
-    def get_or_head_response(self, req, resp_headers, resp_iter):
+    def convert_segment_listing(self, resp_headers, resp_iter):
+        """
+        Converts the manifest data to match with the format
+        that was put in through ?multipart-manifest=put
+
+        :param resp_headers: response headers
+        :param resp_iter: a response iterable
+        """
+        segments = self._get_manifest_read(resp_iter)
+
+        for seg_dict in segments:
+            seg_dict.pop('content_type', None)
+            seg_dict.pop('last_modified', None)
+            seg_dict.pop('sub_slo', None)
+            seg_dict['path'] = seg_dict.pop('name', None)
+            seg_dict['size_bytes'] = seg_dict.pop('bytes', None)
+            seg_dict['etag'] = seg_dict.pop('hash', None)
+
+        json_data = json.dumps(segments)  # convert to string
+        if six.PY3:
+            json_data = json_data.encode('utf-8')
+
+        new_headers = []
+        for header, value in resp_headers:
+            if header.lower() == 'content-length':
+                new_headers.append(('Content-Length',
+                                    len(json_data)))
+            else:
+                new_headers.append((header, value))
+        self._response_headers = new_headers
+
+        return [json_data]
+
+    def _get_manifest_read(self, resp_iter):
         with closing_if_possible(resp_iter):
             resp_body = ''.join(resp_iter)
         try:
@@ -615,6 +656,11 @@ class SloGetContext(WSGIContext):
         except ValueError:
             segments = []
 
+        return segments
+
+    def get_or_head_response(self, req, resp_headers, resp_iter):
+        segments = self._get_manifest_read(resp_iter)
+
         etag = md5()
         content_length = 0
         for seg_dict in segments:
@@ -662,10 +708,17 @@ class SloGetContext(WSGIContext):
         plain_listing_iter = self._segment_listing_iterator(
             req, ver, account, segments)
 
+        def is_small_segment((seg_dict, start_byte, end_byte)):
+            start = 0 if start_byte is None else start_byte
+            end = int(seg_dict['bytes']) - 1 if end_byte is None else end_byte
+            is_small = (end - start + 1) < self.slo.rate_limit_under_size
+            return is_small
+
         ratelimited_listing_iter = RateLimitedIterator(
             plain_listing_iter,
             self.slo.rate_limit_segments_per_sec,
-            limit_after=self.slo.rate_limit_after_segment)
+            limit_after=self.slo.rate_limit_after_segment,
+            ratelimit_if=is_small_segment)
 
         # self._segment_listing_iterator gives us 3-tuples of (segment dict,
         # start byte, end byte), but SegmentedIterable wants (obj path, etag,
@@ -716,7 +769,7 @@ class StaticLargeObject(object):
     :param conf: The configuration dict for the middleware.
     """
 
-    def __init__(self, app, conf, min_segment_size=DEFAULT_MIN_SEGMENT_SIZE,
+    def __init__(self, app, conf,
                  max_manifest_segments=DEFAULT_MAX_MANIFEST_SEGMENTS,
                  max_manifest_size=DEFAULT_MAX_MANIFEST_SIZE):
         self.conf = conf
@@ -724,12 +777,13 @@ class StaticLargeObject(object):
         self.logger = get_logger(conf, log_route='slo')
         self.max_manifest_segments = max_manifest_segments
         self.max_manifest_size = max_manifest_size
-        self.min_segment_size = min_segment_size
         self.max_get_time = int(self.conf.get('max_get_time', 86400))
+        self.rate_limit_under_size = int(self.conf.get(
+            'rate_limit_under_size', DEFAULT_RATE_LIMIT_UNDER_SIZE))
         self.rate_limit_after_segment = int(self.conf.get(
             'rate_limit_after_segment', '10'))
         self.rate_limit_segments_per_sec = int(self.conf.get(
-            'rate_limit_segments_per_sec', '0'))
+            'rate_limit_segments_per_sec', '1'))
         self.bulk_deleter = Bulk(app, {}, logger=self.logger)
 
     def handle_multipart_get_or_head(self, req, start_response):
@@ -783,7 +837,7 @@ class StaticLargeObject(object):
             raise HTTPLengthRequired(request=req)
         parsed_data = parse_and_validate_input(
             req.body_file.read(self.max_manifest_size),
-            req.path, self.min_segment_size)
+            req.path)
         problem_segments = []
 
         if len(parsed_data) > self.max_manifest_segments:
@@ -812,6 +866,7 @@ class StaticLargeObject(object):
             new_env['CONTENT_LENGTH'] = 0
             new_env['HTTP_USER_AGENT'] = \
                 '%s MultipartPUT' % req.environ.get('HTTP_USER_AGENT')
+
             if obj_path != last_obj_path:
                 last_obj_path = obj_path
                 head_seg_resp = \
@@ -840,12 +895,10 @@ class StaticLargeObject(object):
                         seg_dict['range'] = '%d-%d' % (rng[0], rng[1] - 1)
                         segment_length = rng[1] - rng[0]
 
-                if segment_length < self.min_segment_size and \
-                        index < len(parsed_data) - 1:
+                if segment_length < 1:
                     problem_segments.append(
                         [quote(obj_name),
-                         'Too small; each segment, except the last, must be '
-                         'at least %d bytes.' % self.min_segment_size])
+                         'Too small; each segment must be at least 1 byte.'])
                 total_size += segment_length
                 if seg_dict['size_bytes'] is not None and \
                         seg_dict['size_bytes'] != head_seg_resp.content_length:
@@ -1045,18 +1098,17 @@ def filter_factory(global_conf, **local_conf):
                                          DEFAULT_MAX_MANIFEST_SEGMENTS))
     max_manifest_size = int(conf.get('max_manifest_size',
                                      DEFAULT_MAX_MANIFEST_SIZE))
-    min_segment_size = int(conf.get('min_segment_size',
-                                    DEFAULT_MIN_SEGMENT_SIZE))
 
     register_swift_info('slo',
                         max_manifest_segments=max_manifest_segments,
                         max_manifest_size=max_manifest_size,
-                        min_segment_size=min_segment_size)
+                        # this used to be configurable; report it as 1 for
+                        # clients that might still care
+                        min_segment_size=1)
 
     def slo_filter(app):
         return StaticLargeObject(
             app, conf,
             max_manifest_segments=max_manifest_segments,
-            max_manifest_size=max_manifest_size,
-            min_segment_size=min_segment_size)
+            max_manifest_size=max_manifest_size)
     return slo_filter
