@@ -41,10 +41,11 @@ from swift.common.exceptions import ConnectionTimeout
 from swift.common.http import HTTP_NOT_FOUND, is_success
 from swift.common.storage_policy import POLICIES
 from swift.common.base_storage_server import BaseStorageServer
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.swob import HTTPAccepted, HTTPBadRequest, HTTPConflict, \
     HTTPCreated, HTTPInternalServerError, HTTPNoContent, HTTPNotFound, \
     HTTPPreconditionFailed, HTTPMethodNotAllowed, Request, Response, \
-    HTTPInsufficientStorage, HTTPException, HeaderKeyDict
+    HTTPInsufficientStorage, HTTPException
 
 
 def gen_resp_headers(info, is_deleted=False):
@@ -251,7 +252,7 @@ class ContainerController(BaseStorageServer):
             self.sync_store.update_sync_store(broker)
         except Exception:
             self.logger.exception('Failed to update sync_store %s during %s' %
-                                  broker.db_file, method)
+                                  (broker.db_file, method))
 
     @public
     @timing_stats()
@@ -368,7 +369,9 @@ class ContainerController(BaseStorageServer):
                               int(req.headers['x-size']),
                               req.headers['x-content-type'],
                               req.headers['x-etag'], 0,
-                              obj_policy_index)
+                              obj_policy_index,
+                              req.headers.get('x-content-type-timestamp'),
+                              req.headers.get('x-meta-timestamp'))
             return HTTPCreated(request=req)
         else:   # put container
             if requested_policy_index is None:
