@@ -18,7 +18,7 @@ import os
 import uuid
 from swift import gettext_ as _
 from time import ctime, time
-from random import choice, random, shuffle
+from random import choice, random
 from struct import unpack_from
 
 from eventlet import sleep, Timeout
@@ -29,17 +29,17 @@ from swift.container.backend import ContainerBroker
 from swift.container.sync_store import ContainerSyncStore
 from swift.common.container_sync_realms import ContainerSyncRealms
 from swift.common.internal_client import (
-    delete_object, put_object, InternalClient, UnexpectedResponse)
+    delete_object, put_object, head_object,
+    InternalClient, UnexpectedResponse)
 from swift.common.exceptions import ClientException
 from swift.common.ring import Ring
 from swift.common.ring.utils import is_local_device
 from swift.common.utils import (
     clean_content_type, config_true_value,
     FileLikeIter, get_logger, hash_path, quote, urlparse, validate_sync_to,
-    whataremyips, Timestamp)
+    whataremyips, Timestamp, decode_timestamps)
 from swift.common.daemon import Daemon
 from swift.common.http import HTTP_UNAUTHORIZED, HTTP_NOT_FOUND
-from swift.common.storage_policy import POLICIES
 from swift.common.wsgi import ConfigString
 
 
@@ -65,7 +65,7 @@ ic_conf_body = """
 # log_udp_port = 514
 #
 # You can enable StatsD logging here:
-# log_statsd_host = localhost
+# log_statsd_host =
 # log_statsd_port = 8125
 # log_statsd_default_sample_rate = 1.0
 # log_statsd_sample_rate_factor = 1.0
@@ -100,13 +100,6 @@ class ContainerSync(Daemon):
     If they exist, newer rows since the last sync will trigger PUTs or DELETEs
     to the other container.
 
-    .. note::
-
-        Container sync will sync object POSTs only if the proxy server is set
-        to use "object_post_as_copy = true" which is the default. So-called
-        fast object posts, "object_post_as_copy = false" do not update the
-        container listings and therefore can't be detected for synchronization.
-
     The actual syncing is slightly more complicated to make use of the three
     (or number-of-replicas) main nodes for a container without each trying to
     do the exact same work but also without missing work if one node happens to
@@ -170,7 +163,7 @@ class ContainerSync(Daemon):
         #: running wild on near empty systems.
         self.interval = int(conf.get('interval', 300))
         #: Maximum amount of time to spend syncing a container before moving on
-        #: to the next one. If a conatiner sync hasn't finished in this time,
+        #: to the next one. If a container sync hasn't finished in this time,
         #: it'll just be resumed next scan.
         self.container_time = int(conf.get('container_time', 60))
         #: ContainerSyncCluster instance for validating sync-to values.
@@ -238,15 +231,6 @@ class ContainerSync(Daemon):
                 _('Unable to load internal client from config: %r (%s)') %
                 (internal_client_conf_path, err))
 
-    def get_object_ring(self, policy_idx):
-        """
-        Get the ring object to use based on its policy.
-
-        :policy_idx: policy index as defined in swift.conf
-        :returns: appropriate ring object
-        """
-        return POLICIES.get_object_ring(policy_idx, self.swift_dir)
-
     def run_forever(self, *args, **kwargs):
         """
         Runs container sync scans until stopped.
@@ -364,8 +348,6 @@ class ContainerSync(Daemon):
                     row = rows[0]
                     if row['ROWID'] > sync_point1:
                         break
-                    key = hash_path(info['account'], info['container'],
-                                    row['name'], raw_digest=True)
                     # This node will only initially sync out one third of the
                     # objects (if 3 replicas, 1/4 if 4, etc.) and will skip
                     # problematic rows as needed in case of faults.
@@ -408,10 +390,84 @@ class ContainerSync(Daemon):
             self.logger.exception(_('ERROR Syncing %s'),
                                   broker if broker else path)
 
+    def _update_sync_to_headers(self, name, sync_to, user_key,
+                                realm, realm_key, method, headers):
+        """
+        Updates container sync headers
+
+        :param name: The name of the object
+        :param sync_to: The URL to the remote container.
+        :param user_key: The X-Container-Sync-Key to use when sending requests
+                         to the other container.
+        :param realm: The realm from self.realms_conf, if there is one.
+            If None, fallback to using the older allowed_sync_hosts
+            way of syncing.
+        :param realm_key: The realm key from self.realms_conf, if there
+            is one. If None, fallback to using the older
+            allowed_sync_hosts way of syncing.
+        :param method: HTTP method to create sig with
+        :param headers: headers to update with container sync headers
+        """
+        if realm and realm_key:
+            nonce = uuid.uuid4().hex
+            path = urlparse(sync_to).path + '/' + quote(name)
+            sig = self.realms_conf.get_sig(method, path,
+                                           headers.get('x-timestamp', 0),
+                                           nonce, realm_key,
+                                           user_key)
+            headers['x-container-sync-auth'] = '%s %s %s' % (realm,
+                                                             nonce,
+                                                             sig)
+        else:
+            headers['x-container-sync-key'] = user_key
+
+    def _object_in_remote_container(self, name, sync_to, user_key,
+                                    realm, realm_key, timestamp):
+        """
+        Performs head object on remote to eliminate extra remote put and
+        local get object calls
+
+        :param name: The name of the object in the updated row in the local
+                     database triggering the sync update.
+        :param sync_to: The URL to the remote container.
+        :param user_key: The X-Container-Sync-Key to use when sending requests
+                         to the other container.
+        :param realm: The realm from self.realms_conf, if there is one.
+            If None, fallback to using the older allowed_sync_hosts
+            way of syncing.
+        :param realm_key: The realm key from self.realms_conf, if there
+            is one. If None, fallback to using the older
+            allowed_sync_hosts way of syncing.
+        :param timestamp: last modified date of local object
+        :returns: True if object already exists in remote
+        """
+        headers = {'x-timestamp': timestamp.internal}
+        self._update_sync_to_headers(name, sync_to, user_key, realm,
+                                     realm_key, 'HEAD', headers)
+        try:
+            metadata, _ = head_object(sync_to, name=name,
+                                      headers=headers,
+                                      proxy=self.select_http_proxy(),
+                                      logger=self.logger,
+                                      retries=0)
+            remote_ts = Timestamp(metadata.get('x-timestamp', 0))
+            self.logger.debug("remote obj timestamp %s local obj %s" %
+                              (timestamp.internal, remote_ts.internal))
+            if timestamp <= remote_ts:
+                return True
+            # Object in remote should be updated
+            return False
+        except ClientException as http_err:
+            # Object not in remote
+            if http_err.http_status == 404:
+                return False
+            raise http_err
+
     def container_sync_row(self, row, sync_to, user_key, broker, info,
                            realm, realm_key):
         """
         Sends the update the row indicates to the sync_to container.
+        Update can be either delete or put.
 
         :param row: The updated row in the local database triggering the sync
                     update.
@@ -431,20 +487,17 @@ class ContainerSync(Daemon):
         """
         try:
             start_time = time()
+            # extract last modified time from the created_at value
+            ts_data, ts_ctype, ts_meta = decode_timestamps(
+                row['created_at'])
             if row['deleted']:
+                # when sync'ing a deleted object, use ts_data - this is the
+                # timestamp of the source tombstone
                 try:
-                    headers = {'x-timestamp': row['created_at']}
-                    if realm and realm_key:
-                        nonce = uuid.uuid4().hex
-                        path = urlparse(sync_to).path + '/' + quote(
-                            row['name'])
-                        sig = self.realms_conf.get_sig(
-                            'DELETE', path, headers['x-timestamp'], nonce,
-                            realm_key, user_key)
-                        headers['x-container-sync-auth'] = '%s %s %s' % (
-                            realm, nonce, sig)
-                    else:
-                        headers['x-container-sync-key'] = user_key
+                    headers = {'x-timestamp': ts_data.internal}
+                    self._update_sync_to_headers(row['name'], sync_to,
+                                                 user_key, realm, realm_key,
+                                                 'DELETE', headers)
                     delete_object(sync_to, name=row['name'], headers=headers,
                                   proxy=self.select_http_proxy(),
                                   logger=self.logger,
@@ -456,35 +509,30 @@ class ContainerSync(Daemon):
                 self.logger.increment('deletes')
                 self.logger.timing_since('deletes.timing', start_time)
             else:
-                part, nodes = \
-                    self.get_object_ring(info['storage_policy_index']). \
-                    get_nodes(info['account'], info['container'],
-                              row['name'])
-                shuffle(nodes)
+                # when sync'ing a live object, use ts_meta - this is the time
+                # at which the source object was last modified by a PUT or POST
+                if self._object_in_remote_container(row['name'],
+                                                    sync_to, user_key, realm,
+                                                    realm_key, ts_meta):
+                    return True
                 exc = None
-                looking_for_timestamp = Timestamp(row['created_at'])
-                timestamp = -1
-                headers = body = None
                 # look up for the newest one
                 headers_out = {'X-Newest': True,
                                'X-Backend-Storage-Policy-Index':
                                str(info['storage_policy_index'])}
                 try:
-                    source_obj_status, source_obj_info, source_obj_iter = \
+                    source_obj_status, headers, body = \
                         self.swift.get_object(info['account'],
                                               info['container'], row['name'],
                                               headers=headers_out,
                                               acceptable_statuses=(2, 4))
 
                 except (Exception, UnexpectedResponse, Timeout) as err:
-                    source_obj_info = {}
-                    source_obj_iter = None
+                    headers = {}
+                    body = None
                     exc = err
-                timestamp = Timestamp(source_obj_info.get(
-                                      'x-timestamp', 0))
-                headers = source_obj_info
-                body = source_obj_iter
-                if timestamp < looking_for_timestamp:
+                timestamp = Timestamp(headers.get('x-timestamp', 0))
+                if timestamp < ts_meta:
                     if exc:
                         raise exc
                     raise Exception(
@@ -501,17 +549,8 @@ class ContainerSync(Daemon):
                 if 'content-type' in headers:
                     headers['content-type'] = clean_content_type(
                         headers['content-type'])
-                headers['x-timestamp'] = row['created_at']
-                if realm and realm_key:
-                    nonce = uuid.uuid4().hex
-                    path = urlparse(sync_to).path + '/' + quote(row['name'])
-                    sig = self.realms_conf.get_sig(
-                        'PUT', path, headers['x-timestamp'], nonce, realm_key,
-                        user_key)
-                    headers['x-container-sync-auth'] = '%s %s %s' % (
-                        realm, nonce, sig)
-                else:
-                    headers['x-container-sync-key'] = user_key
+                self._update_sync_to_headers(row['name'], sync_to, user_key,
+                                             realm, realm_key, 'PUT', headers)
                 put_object(sync_to, name=row['name'], headers=headers,
                            contents=FileLikeIter(body),
                            proxy=self.select_http_proxy(), logger=self.logger,
