@@ -16,7 +16,7 @@
 
 from os import listdir
 from os.path import join as path_join
-from unittest import main, TestCase
+from unittest import main
 from uuid import uuid4
 
 from eventlet import GreenPool, Timeout
@@ -27,8 +27,8 @@ from swiftclient import client
 from swift.common import direct_client
 from swift.common.exceptions import ClientException
 from swift.common.utils import hash_path, readconf
-from test.probe.common import get_to_final_state, kill_nonprimary_server, \
-    kill_server, kill_servers, reset_environment, start_server
+from test.probe.common import kill_nonprimary_server, \
+    kill_server, ReplProbeTest, start_server
 
 eventlet.monkey_patch(all=False, socket=True)
 
@@ -40,81 +40,93 @@ def get_db_file_path(obj_dir):
             return path_join(obj_dir, filename)
 
 
-class TestContainerFailures(TestCase):
-
-    def setUp(self):
-        (self.pids, self.port2server, self.account_ring, self.container_ring,
-         self.object_ring, self.policy, self.url, self.token,
-         self.account, self.configs) = reset_environment()
-
-    def tearDown(self):
-        kill_servers(self.port2server, self.pids)
+class TestContainerFailures(ReplProbeTest):
 
     def test_one_node_fails(self):
         # Create container1
-        # Kill container1 servers excepting two of the primaries
-        # Delete container1
-        # Restart other container1 primary server
-        # Create container1/object1 (allowed because at least server thinks the
-        #   container exists)
-        # Get to a final state
-        # Assert all container1 servers indicate container1 is alive and
-        #   well with object1
-        # Assert account level also indicates container1 is alive and
-        #   well with object1
         container1 = 'container-%s' % uuid4()
         cpart, cnodes = self.container_ring.get_nodes(self.account, container1)
         client.put_container(self.url, self.token, container1)
-        kill_nonprimary_server(cnodes, self.port2server, self.pids)
-        kill_server(cnodes[0]['port'], self.port2server, self.pids)
+
+        # Kill container1 servers excepting two of the primaries
+        kill_nonprimary_server(cnodes, self.ipport2server, self.pids)
+        kill_server((cnodes[0]['ip'], cnodes[0]['port']),
+                    self.ipport2server, self.pids)
+
+        # Delete container1
         client.delete_container(self.url, self.token, container1)
-        start_server(cnodes[0]['port'], self.port2server, self.pids)
+
+        # Restart other container1 primary server
+        start_server((cnodes[0]['ip'], cnodes[0]['port']),
+                     self.ipport2server, self.pids)
+
+        # Create container1/object1 (allowed because at least server thinks the
+        #   container exists)
         client.put_object(self.url, self.token, container1, 'object1', '123')
-        get_to_final_state()
+
+        # Get to a final state
+        self.get_to_final_state()
+
+        # Assert all container1 servers indicate container1 is alive and
+        #   well with object1
         for cnode in cnodes:
-            self.assertEquals(
+            self.assertEqual(
                 [o['name'] for o in direct_client.direct_get_container(
                     cnode, cpart, self.account, container1)[1]],
                 ['object1'])
+
+        # Assert account level also indicates container1 is alive and
+        #   well with object1
         headers, containers = client.get_account(self.url, self.token)
-        self.assertEquals(headers['x-account-container-count'], '1')
-        self.assertEquals(headers['x-account-object-count'], '1')
-        self.assertEquals(headers['x-account-bytes-used'], '3')
+        self.assertEqual(headers['x-account-container-count'], '1')
+        self.assertEqual(headers['x-account-object-count'], '1')
+        self.assertEqual(headers['x-account-bytes-used'], '3')
 
     def test_two_nodes_fail(self):
         # Create container1
-        # Kill container1 servers excepting one of the primaries
-        # Delete container1 directly to the one primary still up
-        # Restart other container1 servers
-        # Get to a final state
-        # Assert all container1 servers indicate container1 is gone (happens
-        #   because the one node that knew about the delete replicated to the
-        #   others.)
-        # Assert account level also indicates container1 is gone
         container1 = 'container-%s' % uuid4()
         cpart, cnodes = self.container_ring.get_nodes(self.account, container1)
         client.put_container(self.url, self.token, container1)
-        cnp_port = kill_nonprimary_server(cnodes, self.port2server, self.pids)
-        kill_server(cnodes[0]['port'], self.port2server, self.pids)
-        kill_server(cnodes[1]['port'], self.port2server, self.pids)
+
+        # Kill container1 servers excepting one of the primaries
+        cnp_ipport = kill_nonprimary_server(cnodes, self.ipport2server,
+                                            self.pids)
+        kill_server((cnodes[0]['ip'], cnodes[0]['port']),
+                    self.ipport2server, self.pids)
+        kill_server((cnodes[1]['ip'], cnodes[1]['port']),
+                    self.ipport2server, self.pids)
+
+        # Delete container1 directly to the one primary still up
         direct_client.direct_delete_container(cnodes[2], cpart, self.account,
                                               container1)
-        start_server(cnodes[0]['port'], self.port2server, self.pids)
-        start_server(cnodes[1]['port'], self.port2server, self.pids)
-        start_server(cnp_port, self.port2server, self.pids)
-        get_to_final_state()
+
+        # Restart other container1 servers
+        start_server((cnodes[0]['ip'], cnodes[0]['port']),
+                     self.ipport2server, self.pids)
+        start_server((cnodes[1]['ip'], cnodes[1]['port']),
+                     self.ipport2server, self.pids)
+        start_server(cnp_ipport, self.ipport2server, self.pids)
+
+        # Get to a final state
+        self.get_to_final_state()
+
+        # Assert all container1 servers indicate container1 is gone (happens
+        #   because the one node that knew about the delete replicated to the
+        #   others.)
         for cnode in cnodes:
-            exc = None
             try:
                 direct_client.direct_get_container(cnode, cpart, self.account,
                                                    container1)
             except ClientException as err:
-                exc = err
-            self.assertEquals(exc.http_status, 404)
+                self.assertEqual(err.http_status, 404)
+            else:
+                self.fail("Expected ClientException but didn't get it")
+
+        # Assert account level also indicates container1 is gone
         headers, containers = client.get_account(self.url, self.token)
-        self.assertEquals(headers['x-account-container-count'], '0')
-        self.assertEquals(headers['x-account-object-count'], '0')
-        self.assertEquals(headers['x-account-bytes-used'], '0')
+        self.assertEqual(headers['x-account-container-count'], '0')
+        self.assertEqual(headers['x-account-object-count'], '0')
+        self.assertEqual(headers['x-account-bytes-used'], '0')
 
     def _get_container_db_files(self, container):
         opart, onodes = self.container_ring.get_nodes(self.account, container)
@@ -145,12 +157,12 @@ class TestContainerFailures(TestCase):
                 db_conn.execute('begin exclusive transaction')
                 db_conns.append(db_conn)
             if catch_503:
-                exc = None
                 try:
                     client.delete_container(self.url, self.token, container)
                 except client.ClientException as err:
-                    exc = err
-                self.assertEquals(exc.http_status, 503)
+                    self.assertEqual(err.http_status, 503)
+                else:
+                    self.fail("Expected ClientException but didn't get it")
             else:
                 client.delete_container(self.url, self.token, container)
 
