@@ -25,7 +25,7 @@ from mock import patch, call
 from shutil import rmtree, copy
 from tempfile import mkdtemp, NamedTemporaryFile
 import mock
-import simplejson
+import json
 
 from swift.container.backend import DATADIR
 from swift.common import db_replicator
@@ -266,10 +266,15 @@ class TestDBReplicator(unittest.TestCase):
         db_replicator.ring = FakeRing()
         self.delete_db_calls = []
         self._patchers = []
+        # recon cache path
+        self.recon_cache = mkdtemp()
+        rmtree(self.recon_cache, ignore_errors=1)
+        os.mkdir(self.recon_cache)
 
     def tearDown(self):
         for patcher in self._patchers:
             patcher.stop()
+        rmtree(self.recon_cache, ignore_errors=1)
 
     def _patch(self, patching_fn, *args, **kwargs):
         patcher = patching_fn(*args, **kwargs)
@@ -280,6 +285,11 @@ class TestDBReplicator(unittest.TestCase):
     def stub_delete_db(self, broker):
         self.delete_db_calls.append('/path/to/file')
 
+    def test_creation(self):
+        # later config should be extended to assert more config options
+        replicator = TestReplicator({'node_timeout': '3.5'})
+        self.assertEqual(replicator.node_timeout, 3.5)
+
     def test_repl_connection(self):
         node = {'replication_ip': '127.0.0.1', 'replication_port': 80,
                 'device': 'sdb1'}
@@ -287,8 +297,8 @@ class TestDBReplicator(unittest.TestCase):
                                             logging.getLogger())
 
         def req(method, path, body, headers):
-            self.assertEquals(method, 'REPLICATE')
-            self.assertEquals(headers['Content-Type'], 'application/json')
+            self.assertEqual(method, 'REPLICATE')
+            self.assertEqual(headers['Content-Type'], 'application/json')
 
         class Resp(object):
             def read(self):
@@ -296,21 +306,21 @@ class TestDBReplicator(unittest.TestCase):
         resp = Resp()
         conn.request = req
         conn.getresponse = lambda *args: resp
-        self.assertEquals(conn.replicate(1, 2, 3), resp)
+        self.assertEqual(conn.replicate(1, 2, 3), resp)
 
         def other_req(method, path, body, headers):
             raise Exception('blah')
         conn.request = other_req
-        self.assertEquals(conn.replicate(1, 2, 3), None)
+        self.assertEqual(conn.replicate(1, 2, 3), None)
 
     def test_rsync_file(self):
         replicator = TestReplicator({})
         with _mock_process(-1):
-            self.assertEquals(
+            self.assertEqual(
                 False,
                 replicator._rsync_file('/some/file', 'remote:/some/file'))
         with _mock_process(0):
-            self.assertEquals(
+            self.assertEqual(
                 True,
                 replicator._rsync_file('/some/file', 'remote:/some/file'))
 
@@ -414,11 +424,11 @@ class TestDBReplicator(unittest.TestCase):
                             different_region=False):
                 self_._rsync_file_call_count += 1
                 if self_._rsync_file_call_count == 1:
-                    self.assertEquals(True, whole_file)
-                    self.assertEquals(False, self_.broker.locked)
+                    self.assertEqual(True, whole_file)
+                    self.assertEqual(False, self_.broker.locked)
                 elif self_._rsync_file_call_count == 2:
-                    self.assertEquals(False, whole_file)
-                    self.assertEquals(True, self_.broker.locked)
+                    self.assertEqual(False, whole_file)
+                    self.assertEqual(True, self_.broker.locked)
                 else:
                     raise RuntimeError('_rsync_file() called too many times')
                 return True
@@ -430,7 +440,7 @@ class TestDBReplicator(unittest.TestCase):
             fake_device = {'ip': '127.0.0.1', 'replication_ip': '127.0.0.1',
                            'device': 'sda1'}
             replicator._rsync_db(broker, fake_device, ReplHttp(), 'abcd')
-            self.assertEquals(2, replicator._rsync_file_call_count)
+            self.assertEqual(2, replicator._rsync_file_call_count)
 
         # with new mtime
         with patch('os.path.exists', lambda *args: False):
@@ -441,26 +451,46 @@ class TestDBReplicator(unittest.TestCase):
                                'replication_ip': '127.0.0.1',
                                'device': 'sda1'}
                 replicator._rsync_db(broker, fake_device, ReplHttp(), 'abcd')
-                self.assertEquals(2, replicator._rsync_file_call_count)
+                self.assertEqual(2, replicator._rsync_file_call_count)
 
     def test_in_sync(self):
         replicator = TestReplicator({})
-        self.assertEquals(replicator._in_sync(
+        self.assertEqual(replicator._in_sync(
             {'id': 'a', 'point': 0, 'max_row': 0, 'hash': 'b'},
             {'id': 'a', 'point': -1, 'max_row': 0, 'hash': 'b'},
             FakeBroker(), -1), True)
-        self.assertEquals(replicator._in_sync(
+        self.assertEqual(replicator._in_sync(
             {'id': 'a', 'point': -1, 'max_row': 0, 'hash': 'b'},
             {'id': 'a', 'point': -1, 'max_row': 10, 'hash': 'b'},
             FakeBroker(), -1), True)
-        self.assertEquals(bool(replicator._in_sync(
+        self.assertEqual(bool(replicator._in_sync(
             {'id': 'a', 'point': -1, 'max_row': 0, 'hash': 'c'},
             {'id': 'a', 'point': -1, 'max_row': 10, 'hash': 'd'},
             FakeBroker(), -1)), False)
 
-    def test_run_once(self):
-        replicator = TestReplicator({})
-        replicator.run_once()
+    def test_run_once_no_local_device_in_ring(self):
+        logger = unit.debug_logger('test-replicator')
+        replicator = TestReplicator({'recon_cache_path': self.recon_cache},
+                                    logger=logger)
+        with patch('swift.common.db_replicator.whataremyips',
+                   return_value=['127.0.0.1']):
+            replicator.run_once()
+        expected = [
+            "Can't find itself 127.0.0.1 with port 1000 "
+            "in ring file, not replicating",
+        ]
+        self.assertEqual(expected, logger.get_lines_for_level('error'))
+
+    def test_run_once_with_local_device_in_ring(self):
+        logger = unit.debug_logger('test-replicator')
+        base = 'swift.common.db_replicator.'
+        with patch(base + 'whataremyips', return_value=['1.1.1.1']), \
+                patch(base + 'ring', FakeRingWithNodes()):
+            replicator = TestReplicator({'bind_port': 6000,
+                                         'recon_cache_path': self.recon_cache},
+                                        logger=logger)
+            replicator.run_once()
+        self.assertFalse(logger.get_lines_for_level('error'))
 
     def test_run_once_no_ips(self):
         replicator = TestReplicator({}, logger=unit.FakeLogger())
@@ -483,9 +513,9 @@ class TestDBReplicator(unittest.TestCase):
         self.assertEqual(replicator.port, 6000)
 
         def mock_ismount(path):
-            self.assertEquals(path,
-                              os.path.join(replicator.root,
-                                           replicator.ring.devs[0]['device']))
+            self.assertEqual(path,
+                             os.path.join(replicator.root,
+                                          replicator.ring.devs[0]['device']))
             return False
 
         self._patch(patch.object, db_replicator, 'ismount', mock_ismount)
@@ -504,16 +534,16 @@ class TestDBReplicator(unittest.TestCase):
         self.assertEqual(replicator.port, 6000)
 
         def mock_unlink_older_than(path, mtime):
-            self.assertEquals(path,
-                              os.path.join(replicator.root,
-                                           replicator.ring.devs[0]['device'],
-                                           'tmp'))
+            self.assertEqual(path,
+                             os.path.join(replicator.root,
+                                          replicator.ring.devs[0]['device'],
+                                          'tmp'))
             self.assertTrue(time.time() - replicator.reclaim_age >= mtime)
 
         def mock_spawn_n(fn, part, object_file, node_id):
-            self.assertEquals('123', part)
-            self.assertEquals('/srv/node/sda/c.db', object_file)
-            self.assertEquals(1, node_id)
+            self.assertEqual('123', part)
+            self.assertEqual('/srv/node/sda/c.db', object_file)
+            self.assertEqual(1, node_id)
 
         self._patch(patch.object, db_replicator, 'whataremyips',
                     lambda *a, **kw: ['1.1.1.1'])
@@ -562,7 +592,7 @@ class TestDBReplicator(unittest.TestCase):
         replicator = TestReplicator({})
         replicator.delete_db = self.stub_delete_db
         replicator._replicate_object('0', '/path/to/file', 'node_id')
-        self.assertEquals([], self.delete_db_calls)
+        self.assertEqual([], self.delete_db_calls)
 
     def test_replicate_object_quarantine(self):
         replicator = TestReplicator({})
@@ -574,12 +604,12 @@ class TestDBReplicator(unittest.TestCase):
         def mock_renamer(was, new, fsync=False, cause_colision=False):
             if cause_colision and '-' not in new:
                 raise OSError(errno.EEXIST, "File already exists")
-            self.assertEquals('/a/b/c/d/e', was)
+            self.assertEqual('/a/b/c/d/e', was)
             if '-' in new:
                 self.assertTrue(
                     new.startswith('/a/quarantined/containers/e-'))
             else:
-                self.assertEquals('/a/quarantined/containers/e', new)
+                self.assertEqual('/a/quarantined/containers/e', new)
 
         def mock_renamer_error(was, new, fsync):
             return mock_renamer(was, new, fsync, cause_colision=True)
@@ -598,13 +628,13 @@ class TestDBReplicator(unittest.TestCase):
             replicator._replicate_object('0', '/path/to/file', 'node_id')
         finally:
             replicator.brokerclass.stub_replication_info = None
-        self.assertEquals(['/path/to/file'], self.delete_db_calls)
+        self.assertEqual(['/path/to/file'], self.delete_db_calls)
 
     def test_replicate_object_delete_because_not_shouldbehere(self):
         replicator = TestReplicator({})
         replicator.delete_db = self.stub_delete_db
         replicator._replicate_object('0', '/path/to/file', 'node_id')
-        self.assertEquals(['/path/to/file'], self.delete_db_calls)
+        self.assertEqual(['/path/to/file'], self.delete_db_calls)
 
     def test_replicate_account_out_of_place(self):
         replicator = TestReplicator({}, logger=unit.FakeLogger())
@@ -718,28 +748,28 @@ class TestDBReplicator(unittest.TestCase):
     def test_dispatch_no_arg_pop(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
         response = rpc.dispatch(('a',), 'arg')
-        self.assertEquals('Invalid object type', response.body)
-        self.assertEquals(400, response.status_int)
+        self.assertEqual('Invalid object type', response.body)
+        self.assertEqual(400, response.status_int)
 
     def test_dispatch_drive_not_mounted(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, True)
 
         def mock_ismount(path):
-            self.assertEquals('/drive', path)
+            self.assertEqual('/drive', path)
             return False
 
         self._patch(patch.object, db_replicator, 'ismount', mock_ismount)
 
         response = rpc.dispatch(('drive', 'part', 'hash'), ['method'])
 
-        self.assertEquals('507 drive is not mounted', response.status)
-        self.assertEquals(507, response.status_int)
+        self.assertEqual('507 drive is not mounted', response.status)
+        self.assertEqual(507, response.status_int)
 
     def test_dispatch_unexpected_operation_db_does_not_exist(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
 
         def mock_mkdirs(path):
-            self.assertEquals('/drive/tmp', path)
+            self.assertEqual('/drive/tmp', path)
 
         self._patch(patch.object, db_replicator, 'mkdirs', mock_mkdirs)
 
@@ -748,8 +778,8 @@ class TestDBReplicator(unittest.TestCase):
             mock_os.path.exists.return_value = False
             response = rpc.dispatch(('drive', 'part', 'hash'), ['unexpected'])
 
-        self.assertEquals('404 Not Found', response.status)
-        self.assertEquals(404, response.status_int)
+        self.assertEqual('404 Not Found', response.status)
+        self.assertEqual(404, response.status_int)
 
     def test_dispatch_operation_unexpected(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -757,7 +787,7 @@ class TestDBReplicator(unittest.TestCase):
         self._patch(patch.object, db_replicator, 'mkdirs', lambda *args: True)
 
         def unexpected_method(broker, args):
-            self.assertEquals(FakeBroker, broker.__class__)
+            self.assertEqual(FakeBroker, broker.__class__)
             self.assertEqual(['arg1', 'arg2'], args)
             return 'unexpected-called'
 
@@ -770,7 +800,7 @@ class TestDBReplicator(unittest.TestCase):
                                     ['unexpected', 'arg1', 'arg2'])
             mock_os.path.exists.assert_called_with('/part/ash/hash/hash.db')
 
-        self.assertEquals('unexpected-called', response)
+        self.assertEqual('unexpected-called', response)
 
     def test_dispatch_operation_rsync_then_merge(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -784,10 +814,10 @@ class TestDBReplicator(unittest.TestCase):
                                     ['rsync_then_merge', 'arg1', 'arg2'])
             expected_calls = [call('/part/ash/hash/hash.db'),
                               call('/drive/tmp/arg1')]
-            self.assertEquals(mock_os.path.exists.call_args_list,
-                              expected_calls)
-            self.assertEquals('204 No Content', response.status)
-            self.assertEquals(204, response.status_int)
+            self.assertEqual(mock_os.path.exists.call_args_list,
+                             expected_calls)
+            self.assertEqual('204 No Content', response.status)
+            self.assertEqual(204, response.status_int)
 
     def test_dispatch_operation_complete_rsync(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -801,10 +831,10 @@ class TestDBReplicator(unittest.TestCase):
                                     ['complete_rsync', 'arg1', 'arg2'])
             expected_calls = [call('/part/ash/hash/hash.db'),
                               call('/drive/tmp/arg1')]
-            self.assertEquals(mock_os.path.exists.call_args_list,
-                              expected_calls)
-            self.assertEquals('204 No Content', response.status)
-            self.assertEquals(204, response.status_int)
+            self.assertEqual(mock_os.path.exists.call_args_list,
+                             expected_calls)
+            self.assertEqual('204 No Content', response.status)
+            self.assertEqual(204, response.status_int)
 
     def test_rsync_then_merge_db_does_not_exist(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -815,8 +845,8 @@ class TestDBReplicator(unittest.TestCase):
             response = rpc.rsync_then_merge('drive', '/data/db.db',
                                             ('arg1', 'arg2'))
             mock_os.path.exists.assert_called_with('/data/db.db')
-            self.assertEquals('404 Not Found', response.status)
-            self.assertEquals(404, response.status_int)
+            self.assertEqual('404 Not Found', response.status)
+            self.assertEqual(404, response.status_int)
 
     def test_rsync_then_merge_old_does_not_exist(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -827,17 +857,17 @@ class TestDBReplicator(unittest.TestCase):
             response = rpc.rsync_then_merge('drive', '/data/db.db',
                                             ('arg1', 'arg2'))
             expected_calls = [call('/data/db.db'), call('/drive/tmp/arg1')]
-            self.assertEquals(mock_os.path.exists.call_args_list,
-                              expected_calls)
-            self.assertEquals('404 Not Found', response.status)
-            self.assertEquals(404, response.status_int)
+            self.assertEqual(mock_os.path.exists.call_args_list,
+                             expected_calls)
+            self.assertEqual('404 Not Found', response.status)
+            self.assertEqual(404, response.status_int)
 
     def test_rsync_then_merge_with_objects(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
 
         def mock_renamer(old, new):
-            self.assertEquals('/drive/tmp/arg1', old)
-            self.assertEquals('/data/db.db', new)
+            self.assertEqual('/drive/tmp/arg1', old)
+            self.assertEqual('/data/db.db', new)
 
         self._patch(patch.object, db_replicator, 'renamer', mock_renamer)
 
@@ -846,8 +876,8 @@ class TestDBReplicator(unittest.TestCase):
             mock_os.path.exists.return_value = True
             response = rpc.rsync_then_merge('drive', '/data/db.db',
                                             ['arg1', 'arg2'])
-            self.assertEquals('204 No Content', response.status)
-            self.assertEquals(204, response.status_int)
+            self.assertEqual('204 No Content', response.status)
+            self.assertEqual(204, response.status_int)
 
     def test_complete_rsync_db_does_not_exist(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -858,8 +888,8 @@ class TestDBReplicator(unittest.TestCase):
             response = rpc.complete_rsync('drive', '/data/db.db',
                                           ['arg1', 'arg2'])
             mock_os.path.exists.assert_called_with('/data/db.db')
-            self.assertEquals('404 Not Found', response.status)
-            self.assertEquals(404, response.status_int)
+            self.assertEqual('404 Not Found', response.status)
+            self.assertEqual(404, response.status_int)
 
     def test_complete_rsync_old_file_does_not_exist(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -870,10 +900,10 @@ class TestDBReplicator(unittest.TestCase):
             response = rpc.complete_rsync('drive', '/data/db.db',
                                           ['arg1', 'arg2'])
             expected_calls = [call('/data/db.db'), call('/drive/tmp/arg1')]
-            self.assertEquals(expected_calls,
-                              mock_os.path.exists.call_args_list)
-            self.assertEquals('404 Not Found', response.status)
-            self.assertEquals(404, response.status_int)
+            self.assertEqual(expected_calls,
+                             mock_os.path.exists.call_args_list)
+            self.assertEqual('404 Not Found', response.status)
+            self.assertEqual(404, response.status_int)
 
     def test_complete_rsync_rename(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -881,12 +911,12 @@ class TestDBReplicator(unittest.TestCase):
         def mock_exists(path):
             if path == '/data/db.db':
                 return False
-            self.assertEquals('/drive/tmp/arg1', path)
+            self.assertEqual('/drive/tmp/arg1', path)
             return True
 
         def mock_renamer(old, new):
-            self.assertEquals('/drive/tmp/arg1', old)
-            self.assertEquals('/data/db.db', new)
+            self.assertEqual('/drive/tmp/arg1', old)
+            self.assertEqual('/data/db.db', new)
 
         self._patch(patch.object, db_replicator, 'renamer', mock_renamer)
 
@@ -895,8 +925,8 @@ class TestDBReplicator(unittest.TestCase):
             mock_os.path.exists.side_effect = [False, True]
             response = rpc.complete_rsync('drive', '/data/db.db',
                                           ['arg1', 'arg2'])
-            self.assertEquals('204 No Content', response.status)
-            self.assertEquals(204, response.status_int)
+            self.assertEqual('204 No Content', response.status)
+            self.assertEqual(204, response.status_int)
 
     def test_replicator_sync_with_broker_replication_missing_table(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -908,8 +938,8 @@ class TestDBReplicator(unittest.TestCase):
 
         def mock_quarantine_db(object_file, server_type):
             called.append(True)
-            self.assertEquals(broker.db_file, object_file)
-            self.assertEquals(broker.db_type, server_type)
+            self.assertEqual(broker.db_file, object_file)
+            self.assertEqual(broker.db_type, server_type)
 
         self._patch(patch.object, db_replicator, 'quarantine_db',
                     mock_quarantine_db)
@@ -918,8 +948,8 @@ class TestDBReplicator(unittest.TestCase):
                                      'created_at', 'put_timestamp',
                                      'delete_timestamp', 'metadata'))
 
-        self.assertEquals('404 Not Found', response.status)
-        self.assertEquals(404, response.status_int)
+        self.assertEqual('404 Not Found', response.status)
+        self.assertEqual(404, response.status_int)
         self.assertEqual(called, [True])
         errors = rpc.logger.get_lines_for_level('error')
         self.assertEqual(errors,
@@ -935,14 +965,14 @@ class TestDBReplicator(unittest.TestCase):
                                      'delete_timestamp',
                                      '{"meta1": "data1", "meta2": "data2"}'))
 
-        self.assertEquals({'meta1': 'data1', 'meta2': 'data2'},
-                          broker.metadata)
-        self.assertEquals('created_at', broker.created_at)
-        self.assertEquals('put_timestamp', broker.put_timestamp)
-        self.assertEquals('delete_timestamp', broker.delete_timestamp)
+        self.assertEqual({'meta1': 'data1', 'meta2': 'data2'},
+                         broker.metadata)
+        self.assertEqual('created_at', broker.created_at)
+        self.assertEqual('put_timestamp', broker.put_timestamp)
+        self.assertEqual('delete_timestamp', broker.delete_timestamp)
 
-        self.assertEquals('200 OK', response.status)
-        self.assertEquals(200, response.status_int)
+        self.assertEqual('200 OK', response.status)
+        self.assertEqual(200, response.status_int)
 
     def test_rsync_then_merge(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
@@ -953,14 +983,14 @@ class TestDBReplicator(unittest.TestCase):
         fake_broker = FakeBroker()
         args = ('a', 'b')
         rpc.merge_items(fake_broker, args)
-        self.assertEquals(fake_broker.args, args)
+        self.assertEqual(fake_broker.args, args)
 
     def test_merge_syncs(self):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
         fake_broker = FakeBroker()
         args = ('a', 'b')
         rpc.merge_syncs(fake_broker, args)
-        self.assertEquals(fake_broker.args, (args[0],))
+        self.assertEqual(fake_broker.args, (args[0],))
 
     def test_complete_rsync_with_bad_input(self):
         drive = '/some/root'
@@ -969,10 +999,10 @@ class TestDBReplicator(unittest.TestCase):
         rpc = db_replicator.ReplicatorRpc('/', '/', FakeBroker, False)
         resp = rpc.complete_rsync(drive, db_file, args)
         self.assertTrue(isinstance(resp, HTTPException))
-        self.assertEquals(404, resp.status_int)
+        self.assertEqual(404, resp.status_int)
         resp = rpc.complete_rsync(drive, 'new_db_file', args)
         self.assertTrue(isinstance(resp, HTTPException))
-        self.assertEquals(404, resp.status_int)
+        self.assertEqual(404, resp.status_int)
 
     def test_complete_rsync(self):
         drive = mkdtemp()
@@ -986,7 +1016,7 @@ class TestDBReplicator(unittest.TestCase):
             fp.write('void')
             fp.close
             resp = rpc.complete_rsync(drive, new_file, args)
-            self.assertEquals(204, resp.status_int)
+            self.assertEqual(204, resp.status_int)
         finally:
             rmtree(drive)
 
@@ -1076,7 +1106,7 @@ class TestDBReplicator(unittest.TestCase):
             results = list(db_replicator.roundrobin_datadirs(datadirs))
             # The results show that the .db files are returned, the devices
             # interleaved.
-            self.assertEquals(results, [
+            self.assertEqual(results, [
                 ('123', '/srv/node/sda/containers/123/abc/'
                         '00000000000000000000000000000abc/'
                         '00000000000000000000000000000abc.db', 1),
@@ -1090,7 +1120,7 @@ class TestDBReplicator(unittest.TestCase):
                         '22222222222222222222222222222ghi/'
                         '22222222222222222222222222222ghi.db', 2)])
             # The listdir calls show that we only listdir the dirs
-            self.assertEquals(listdir_calls, [
+            self.assertEqual(listdir_calls, [
                 '/srv/node/sda/containers',
                 '/srv/node/sda/containers/123',
                 '/srv/node/sda/containers/123/abc',
@@ -1105,7 +1135,7 @@ class TestDBReplicator(unittest.TestCase):
                 '/srv/node/sdb/containers/9999'])
             # The isdir calls show that we did ask about the things pretending
             # to be files at various levels.
-            self.assertEquals(isdir_calls, [
+            self.assertEqual(isdir_calls, [
                 '/srv/node/sda/containers/123',
                 '/srv/node/sda/containers/123/abc',
                 ('/srv/node/sda/containers/123/abc/'
@@ -1140,7 +1170,7 @@ class TestDBReplicator(unittest.TestCase):
                 '/srv/node/sdb/containers/9999'])
             # The exists calls are the .db files we looked for as we walked the
             # structure.
-            self.assertEquals(exists_calls, [
+            self.assertEqual(exists_calls, [
                 ('/srv/node/sda/containers/123/abc/'
                  '00000000000000000000000000000abc/'
                  '00000000000000000000000000000abc.db'),
@@ -1154,12 +1184,12 @@ class TestDBReplicator(unittest.TestCase):
                  '22222222222222222222222222222ghi/'
                  '22222222222222222222222222222ghi.db')])
             # Shows that we called shuffle twice, once for each device.
-            self.assertEquals(
+            self.assertEqual(
                 shuffle_calls, [['123', '456', '789', '9999'],
                                 ['123', '456', '789', '9999']])
 
             # Shows that we called removed the two empty partition directories.
-            self.assertEquals(
+            self.assertEqual(
                 rmdir_calls, ['/srv/node/sda/containers/9999',
                               '/srv/node/sdb/containers/9999'])
         finally:
@@ -1187,9 +1217,9 @@ class TestReplToNode(unittest.TestCase):
         db_replicator.ring = FakeRing()
         self.delete_db_calls = []
         self.broker = FakeBroker()
-        self.replicator = TestReplicator({})
+        self.replicator = TestReplicator({'per_diff': 10})
         self.fake_node = {'ip': '127.0.0.1', 'device': 'sda1', 'port': 1000}
-        self.fake_info = {'id': 'a', 'point': -1, 'max_row': 10, 'hash': 'b',
+        self.fake_info = {'id': 'a', 'point': -1, 'max_row': 20, 'hash': 'b',
                           'created_at': 100, 'put_timestamp': 0,
                           'delete_timestamp': 0, 'count': 0,
                           'metadata': {
@@ -1201,10 +1231,10 @@ class TestReplToNode(unittest.TestCase):
         self.replicator._http_connect = lambda *args: self.http
 
     def test_repl_to_node_usync_success(self):
-        rinfo = {"id": 3, "point": -1, "max_row": 5, "hash": "c"}
-        self.http = ReplHttp(simplejson.dumps(rinfo))
+        rinfo = {"id": 3, "point": -1, "max_row": 10, "hash": "c"}
+        self.http = ReplHttp(json.dumps(rinfo))
         local_sync = self.broker.get_sync()
-        self.assertEquals(self.replicator._repl_to_node(
+        self.assertEqual(self.replicator._repl_to_node(
             self.fake_node, self.broker, '0', self.fake_info), True)
         self.replicator._usync_db.assert_has_calls([
             mock.call(max(rinfo['point'], local_sync), self.broker,
@@ -1212,10 +1242,10 @@ class TestReplToNode(unittest.TestCase):
         ])
 
     def test_repl_to_node_rsync_success(self):
-        rinfo = {"id": 3, "point": -1, "max_row": 4, "hash": "c"}
-        self.http = ReplHttp(simplejson.dumps(rinfo))
+        rinfo = {"id": 3, "point": -1, "max_row": 9, "hash": "c"}
+        self.http = ReplHttp(json.dumps(rinfo))
         self.broker.get_sync()
-        self.assertEquals(self.replicator._repl_to_node(
+        self.assertEqual(self.replicator._repl_to_node(
             self.fake_node, self.broker, '0', self.fake_info), True)
         self.replicator.logger.increment.assert_has_calls([
             mock.call.increment('remote_merges')
@@ -1229,17 +1259,17 @@ class TestReplToNode(unittest.TestCase):
         ])
 
     def test_repl_to_node_already_in_sync(self):
-        rinfo = {"id": 3, "point": -1, "max_row": 10, "hash": "b"}
-        self.http = ReplHttp(simplejson.dumps(rinfo))
+        rinfo = {"id": 3, "point": -1, "max_row": 20, "hash": "b"}
+        self.http = ReplHttp(json.dumps(rinfo))
         self.broker.get_sync()
-        self.assertEquals(self.replicator._repl_to_node(
+        self.assertEqual(self.replicator._repl_to_node(
             self.fake_node, self.broker, '0', self.fake_info), True)
-        self.assertEquals(self.replicator._rsync_db.call_count, 0)
-        self.assertEquals(self.replicator._usync_db.call_count, 0)
+        self.assertEqual(self.replicator._rsync_db.call_count, 0)
+        self.assertEqual(self.replicator._usync_db.call_count, 0)
 
     def test_repl_to_node_not_found(self):
         self.http = ReplHttp('{"id": 3, "point": -1}', set_status=404)
-        self.assertEquals(self.replicator._repl_to_node(
+        self.assertEqual(self.replicator._repl_to_node(
             self.fake_node, self.broker, '0', self.fake_info, False), True)
         self.replicator.logger.increment.assert_has_calls([
             mock.call.increment('rsyncs')
@@ -1258,14 +1288,34 @@ class TestReplToNode(unittest.TestCase):
     def test_repl_to_node_300_status(self):
         self.http = ReplHttp('{"id": 3, "point": -1}', set_status=300)
 
-        self.assertEquals(self.replicator._repl_to_node(
+        self.assertEqual(self.replicator._repl_to_node(
             self.fake_node, FakeBroker(), '0', self.fake_info), None)
 
     def test_repl_to_node_not_response(self):
         self.http = mock.Mock(replicate=mock.Mock(return_value=None))
-        self.assertEquals(self.replicator._repl_to_node(
+        self.assertEqual(self.replicator._repl_to_node(
             self.fake_node, FakeBroker(), '0', self.fake_info), False)
 
+    def test_repl_to_node_small_container_always_usync(self):
+        # Tests that a small container that is > 50% out of sync will
+        # still use usync.
+        rinfo = {"id": 3, "point": -1, "hash": "c"}
+
+        # Turn per_diff back to swift's default.
+        self.replicator.per_diff = 1000
+        for r, l in ((5, 20), (40, 100), (450, 1000), (550, 1500)):
+            rinfo['max_row'] = r
+            self.fake_info['max_row'] = l
+            self.replicator._usync_db = mock.Mock(return_value=True)
+            self.http = ReplHttp(json.dumps(rinfo))
+            local_sync = self.broker.get_sync()
+            self.assertEqual(self.replicator._repl_to_node(
+                self.fake_node, self.broker, '0', self.fake_info), True)
+            self.replicator._usync_db.assert_has_calls([
+                mock.call(max(rinfo['point'], local_sync), self.broker,
+                          self.http, rinfo['id'], self.fake_info['id'])
+            ])
+
 
 class FakeHTTPResponse(object):
 
