@@ -12,6 +12,8 @@
 # limitations under the License.
 
 import json
+import numbers
+
 import mock
 import operator
 import time
@@ -29,7 +31,8 @@ from swift.container import reconciler
 from swift.container.server import gen_resp_headers
 from swift.common.direct_client import ClientException
 from swift.common import swob
-from swift.common.utils import split_path, Timestamp
+from swift.common.header_key_dict import HeaderKeyDict
+from swift.common.utils import split_path, Timestamp, encode_timestamps
 
 from test.unit import debug_logger, FakeRing, fake_http_connect
 from test.unit.common.middleware.helpers import FakeSwift
@@ -41,7 +44,7 @@ def timestamp_to_last_modified(timestamp):
 
 
 def container_resp_headers(**kwargs):
-    return swob.HeaderKeyDict(gen_resp_headers(kwargs))
+    return HeaderKeyDict(gen_resp_headers(kwargs))
 
 
 class FakeStoragePolicySwift(object):
@@ -132,12 +135,16 @@ class FakeInternalClient(reconciler.InternalClient):
                         'DELETE', obj_path, swob.HTTPNoContent, {})
                     # container listing entry
                     last_modified = timestamp_to_last_modified(timestamp)
+                    # some tests setup mock listings using floats, some use
+                    # strings, so normalize here
+                    if isinstance(timestamp, numbers.Number):
+                        timestamp = '%f' % timestamp
                     obj_data = {
                         'bytes': 0,
                         # listing data is unicode
                         'name': obj_name.decode('utf-8'),
                         'last_modified': last_modified,
-                        'hash': timestamp,
+                        'hash': timestamp.decode('utf-8'),
                         'content_type': content_type,
                     }
                     container_listing_data.append(obj_data)
@@ -210,6 +217,26 @@ class TestReconcilerUtils(unittest.TestCase):
         self.assertEqual(got['q_record'], 1234.20192)
         self.assertEqual(got['q_op'], 'PUT')
 
+        # the 'hash' field in object listing has the raw 'created_at' value
+        # which could be a composite of timestamps
+        timestamp_str = encode_timestamps(Timestamp(1234.20190),
+                                          Timestamp(1245.20190),
+                                          Timestamp(1256.20190),
+                                          explicit=True)
+        got = reconciler.parse_raw_obj({
+            'name': "1:/AUTH_bob/con/obj",
+            'hash': timestamp_str,
+            'last_modified': timestamp_to_last_modified(1234.20192),
+            'content_type': 'application/x-put',
+        })
+        self.assertEqual(got['q_policy_index'], 1)
+        self.assertEqual(got['account'], 'AUTH_bob')
+        self.assertEqual(got['container'], 'con')
+        self.assertEqual(got['obj'], 'obj')
+        self.assertEqual(got['q_ts'], 1234.20190)
+        self.assertEqual(got['q_record'], 1234.20192)
+        self.assertEqual(got['q_op'], 'PUT')
+
         # negative test
         obj_info = {
             'name': "1:/AUTH_bob/con/obj",
