@@ -22,14 +22,18 @@ import string
 from shutil import rmtree
 from hashlib import md5
 from tempfile import mkdtemp
-from test.unit import FakeLogger, patch_policies, make_timestamp_iter, \
-    DEFAULT_TEST_EC_TYPE
-from swift.obj import auditor
-from swift.obj.diskfile import DiskFile, write_metadata, invalidate_hash, \
-    get_data_dir, DiskFileManager, ECDiskFileManager, AuditLocation
-from swift.common.utils import mkdirs, normalize_timestamp, Timestamp
-from swift.common.storage_policy import ECStoragePolicy, StoragePolicy, \
-    POLICIES
+import textwrap
+from test.unit import (FakeLogger, patch_policies, make_timestamp_iter,
+                       DEFAULT_TEST_EC_TYPE)
+from swift.obj import auditor, replicator
+from swift.obj.diskfile import (
+    DiskFile, write_metadata, invalidate_hash, get_data_dir,
+    DiskFileManager, ECDiskFileManager, AuditLocation, clear_auditor_status,
+    get_auditor_status)
+from swift.common.utils import (
+    mkdirs, normalize_timestamp, Timestamp, readconf)
+from swift.common.storage_policy import (
+    ECStoragePolicy, StoragePolicy, POLICIES)
 
 
 _mocked_policies = [
@@ -40,6 +44,19 @@ _mocked_policies = [
 ]
 
 
+def works_only_once(callable_thing, exception):
+    called = [False]
+
+    def only_once(*a, **kw):
+        if called[0]:
+            raise exception
+        else:
+            called[0] = True
+            return callable_thing(*a, **kw)
+
+    return only_once
+
+
 @patch_policies(_mocked_policies)
 class TestAuditor(unittest.TestCase):
 
@@ -261,6 +278,161 @@ class TestAuditor(unittest.TestCase):
                               policy=POLICIES.legacy))
         self.assertEqual(auditor_worker.errors, 1)
 
+    def test_audit_location_gets_quarantined(self):
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+
+        location = AuditLocation(self.disk_file._datadir, 'sda', '0',
+                                 policy=self.disk_file.policy)
+
+        # instead of a datadir, we'll make a file!
+        mkdirs(os.path.dirname(self.disk_file._datadir))
+        open(self.disk_file._datadir, 'w')
+
+        # after we turn the crank ...
+        auditor_worker.object_audit(location)
+
+        # ... it should get quarantined
+        self.assertFalse(os.path.exists(self.disk_file._datadir))
+        self.assertEqual(1, auditor_worker.quarantines)
+
+    def test_rsync_tempfile_timeout_auto_option(self):
+        # if we don't have access to the replicator config section we'll use
+        # our default
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+        self.assertEqual(auditor_worker.rsync_tempfile_timeout, 86400)
+        # if the rsync_tempfile_timeout option is set explicitly we use that
+        self.conf['rsync_tempfile_timeout'] = '1800'
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+        self.assertEqual(auditor_worker.rsync_tempfile_timeout, 1800)
+        # if we have a real config we can be a little smarter
+        config_path = os.path.join(self.testdir, 'objserver.conf')
+        stub_config = """
+        [object-auditor]
+        rsync_tempfile_timeout = auto
+        """
+        with open(config_path, 'w') as f:
+            f.write(textwrap.dedent(stub_config))
+        # the Daemon loader will hand the object-auditor config to the
+        # auditor who will build the workers from it
+        conf = readconf(config_path, 'object-auditor')
+        auditor_worker = auditor.AuditorWorker(conf, self.logger,
+                                               self.rcache, self.devices)
+        # if there is no object-replicator section we still have to fall back
+        # to default because we can't parse the config for that section!
+        self.assertEqual(auditor_worker.rsync_tempfile_timeout, 86400)
+        stub_config = """
+        [object-replicator]
+        [object-auditor]
+        rsync_tempfile_timeout = auto
+        """
+        with open(os.path.join(self.testdir, 'objserver.conf'), 'w') as f:
+            f.write(textwrap.dedent(stub_config))
+        conf = readconf(config_path, 'object-auditor')
+        auditor_worker = auditor.AuditorWorker(conf, self.logger,
+                                               self.rcache, self.devices)
+        # if the object-replicator section will parse but does not override
+        # the default rsync_timeout we assume the default rsync_timeout value
+        # and add 15mins
+        self.assertEqual(auditor_worker.rsync_tempfile_timeout,
+                         replicator.DEFAULT_RSYNC_TIMEOUT + 900)
+        stub_config = """
+        [DEFAULT]
+        reclaim_age = 1209600
+        [object-replicator]
+        rsync_timeout = 3600
+        [object-auditor]
+        rsync_tempfile_timeout = auto
+        """
+        with open(os.path.join(self.testdir, 'objserver.conf'), 'w') as f:
+            f.write(textwrap.dedent(stub_config))
+        conf = readconf(config_path, 'object-auditor')
+        auditor_worker = auditor.AuditorWorker(conf, self.logger,
+                                               self.rcache, self.devices)
+        # if there is an object-replicator section with a rsync_timeout
+        # configured we'll use that value (3600) + 900
+        self.assertEqual(auditor_worker.rsync_tempfile_timeout, 3600 + 900)
+
+    def test_inprogress_rsync_tempfiles_get_cleaned_up(self):
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+
+        location = AuditLocation(self.disk_file._datadir, 'sda', '0',
+                                 policy=self.disk_file.policy)
+
+        data = 'VERIFY'
+        etag = md5()
+        timestamp = str(normalize_timestamp(time.time()))
+        with self.disk_file.create() as writer:
+            writer.write(data)
+            etag.update(data)
+            metadata = {
+                'ETag': etag.hexdigest(),
+                'X-Timestamp': timestamp,
+                'Content-Length': str(os.fstat(writer._fd).st_size),
+            }
+            writer.put(metadata)
+            writer.commit(Timestamp(timestamp))
+
+        datafilename = None
+        datadir_files = os.listdir(self.disk_file._datadir)
+        for filename in datadir_files:
+            if filename.endswith('.data'):
+                datafilename = filename
+                break
+        else:
+            self.fail('Did not find .data file in %r: %r' %
+                      (self.disk_file._datadir, datadir_files))
+        rsynctempfile_path = os.path.join(self.disk_file._datadir,
+                                          '.%s.9ILVBL' % datafilename)
+        open(rsynctempfile_path, 'w')
+        # sanity check we have an extra file
+        rsync_files = os.listdir(self.disk_file._datadir)
+        self.assertEqual(len(datadir_files) + 1, len(rsync_files))
+
+        # and after we turn the crank ...
+        auditor_worker.object_audit(location)
+
+        # ... we've still got the rsync file
+        self.assertEqual(rsync_files, os.listdir(self.disk_file._datadir))
+
+        # and we'll keep it - depending on the rsync_tempfile_timeout
+        self.assertEqual(auditor_worker.rsync_tempfile_timeout, 86400)
+        self.conf['rsync_tempfile_timeout'] = '3600'
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+        self.assertEqual(auditor_worker.rsync_tempfile_timeout, 3600)
+        now = time.time() + 1900
+        with mock.patch('swift.obj.auditor.time.time',
+                        return_value=now):
+            auditor_worker.object_audit(location)
+        self.assertEqual(rsync_files, os.listdir(self.disk_file._datadir))
+
+        # but *tomorrow* when we run
+        tomorrow = time.time() + 86400
+        with mock.patch('swift.obj.auditor.time.time',
+                        return_value=tomorrow):
+            auditor_worker.object_audit(location)
+
+        # ... we'll totally clean that stuff up!
+        self.assertEqual(datadir_files, os.listdir(self.disk_file._datadir))
+
+        # but if we have some random crazy file in there
+        random_crazy_file_path = os.path.join(self.disk_file._datadir,
+                                              '.random.crazy.file')
+        open(random_crazy_file_path, 'w')
+
+        tomorrow = time.time() + 86400
+        with mock.patch('swift.obj.auditor.time.time',
+                        return_value=tomorrow):
+            auditor_worker.object_audit(location)
+
+        # that's someone elses problem
+        self.assertIn(os.path.basename(random_crazy_file_path),
+                      os.listdir(self.disk_file._datadir))
+
     def test_generic_exception_handling(self):
         auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                self.rcache, self.devices)
@@ -460,6 +632,7 @@ class TestAuditor(unittest.TestCase):
         self.auditor.run_audit(**kwargs)
         self.assertFalse(os.path.isdir(quarantine_path))
         del(kwargs['zero_byte_fps'])
+        clear_auditor_status(self.devices)
         self.auditor.run_audit(**kwargs)
         self.assertTrue(os.path.isdir(quarantine_path))
 
@@ -495,10 +668,20 @@ class TestAuditor(unittest.TestCase):
         self.setup_bad_zero_byte()
         kwargs = {'mode': 'once'}
         kwargs['zero_byte_fps'] = 50
-        self.auditor.run_audit(**kwargs)
+
+        called_args = [0]
+
+        def mock_get_auditor_status(path, logger, audit_type):
+            called_args[0] = audit_type
+            return get_auditor_status(path, logger, audit_type)
+
+        with mock.patch('swift.obj.diskfile.get_auditor_status',
+                        mock_get_auditor_status):
+                self.auditor.run_audit(**kwargs)
         quarantine_path = os.path.join(self.devices,
                                        'sda', 'quarantined', 'objects')
         self.assertTrue(os.path.isdir(quarantine_path))
+        self.assertEqual('ZBF', called_args[0])
 
     def test_object_run_fast_track_zero_check_closed(self):
         rat = [False]
@@ -591,6 +774,9 @@ class TestAuditor(unittest.TestCase):
 
         loop_error = Bogus('exception')
 
+        class LetMeOut(BaseException):
+            pass
+
         class ObjectAuditorMock(object):
             check_args = ()
             check_kwargs = {}
@@ -687,11 +873,13 @@ class TestAuditor(unittest.TestCase):
             self.assertEqual(mocker.wait_called, 1)
 
             my_auditor._sleep = mocker.mock_sleep_continue
+            my_auditor.audit_loop = works_only_once(my_auditor.audit_loop,
+                                                    LetMeOut())
 
             my_auditor.concurrency = 2
             mocker.fork_called = 0
             mocker.wait_called = 0
-            my_auditor.run_once()
+            self.assertRaises(LetMeOut, my_auditor.run_forever)
             # Fork is called no. of devices + (no. of devices)/2 + 1 times
             # since zbf process is forked (no.of devices)/2 + 1 times
             no_devices = len(os.listdir(self.devices))
@@ -704,5 +892,102 @@ class TestAuditor(unittest.TestCase):
             os.fork = was_fork
             os.wait = was_wait
 
+    def test_run_audit_once(self):
+        my_auditor = auditor.ObjectAuditor(dict(devices=self.devices,
+                                                mount_check='false',
+                                                zero_byte_files_per_second=89,
+                                                concurrency=1))
+
+        forked_pids = []
+        next_zbf_pid = [2]
+        next_normal_pid = [1001]
+        outstanding_pids = [[]]
+
+        def fake_fork_child(**kwargs):
+            if len(forked_pids) > 10:
+                # something's gone horribly wrong
+                raise BaseException("forking too much")
+
+            # ZBF pids are all smaller than the normal-audit pids; this way
+            # we can return them first.
+            #
+            # Also, ZBF pids are even and normal-audit pids are odd; this is
+            # so humans seeing this test fail can better tell what's happening.
+            if kwargs.get('zero_byte_fps'):
+                pid = next_zbf_pid[0]
+                next_zbf_pid[0] += 2
+            else:
+                pid = next_normal_pid[0]
+                next_normal_pid[0] += 2
+            outstanding_pids[0].append(pid)
+            forked_pids.append(pid)
+            return pid
+
+        def fake_os_wait():
+            # Smallest pid first; that's ZBF if we have one, else normal
+            outstanding_pids[0].sort()
+            pid = outstanding_pids[0].pop(0)
+            return (pid, 0)   # (pid, status)
+
+        with mock.patch("swift.obj.auditor.os.wait", fake_os_wait), \
+                mock.patch.object(my_auditor, 'fork_child', fake_fork_child), \
+                mock.patch.object(my_auditor, '_sleep', lambda *a: None):
+            my_auditor.run_once()
+
+        self.assertEqual(sorted(forked_pids), [2, 1001])
+
+    def test_run_parallel_audit_once(self):
+        my_auditor = auditor.ObjectAuditor(
+            dict(devices=self.devices, mount_check='false',
+                 zero_byte_files_per_second=89, concurrency=2))
+
+        # ZBF pids are smaller than the normal-audit pids; this way we can
+        # return them first from our mocked os.wait().
+        #
+        # Also, ZBF pids are even and normal-audit pids are odd; this is so
+        # humans seeing this test fail can better tell what's happening.
+        forked_pids = []
+        next_zbf_pid = [2]
+        next_normal_pid = [1001]
+        outstanding_pids = [[]]
+
+        def fake_fork_child(**kwargs):
+            if len(forked_pids) > 10:
+                # something's gone horribly wrong; try not to hang the test
+                # run because of it
+                raise BaseException("forking too much")
+
+            if kwargs.get('zero_byte_fps'):
+                pid = next_zbf_pid[0]
+                next_zbf_pid[0] += 2
+            else:
+                pid = next_normal_pid[0]
+                next_normal_pid[0] += 2
+            outstanding_pids[0].append(pid)
+            forked_pids.append(pid)
+            return pid
+
+        def fake_os_wait():
+            if not outstanding_pids[0]:
+                raise BaseException("nobody waiting")
+
+            # ZBF auditor finishes first
+            outstanding_pids[0].sort()
+            pid = outstanding_pids[0].pop(0)
+            return (pid, 0)   # (pid, status)
+
+        # make sure we've got enough devs that the ZBF auditor can finish
+        # before all the normal auditors have been started
+        mkdirs(os.path.join(self.devices, 'sdc'))
+        mkdirs(os.path.join(self.devices, 'sdd'))
+
+        with mock.patch("swift.obj.auditor.os.wait", fake_os_wait), \
+                mock.patch.object(my_auditor, 'fork_child', fake_fork_child), \
+                mock.patch.object(my_auditor, '_sleep', lambda *a: None):
+            my_auditor.run_once()
+
+        self.assertEqual(sorted(forked_pids), [2, 1001, 1003, 1005, 1007])
+
+
 if __name__ == '__main__':
     unittest.main()
